<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/**
 * Game AdminPanel (АдминПанель)
 *
 * 
 *
 * @package		Game AdminPanel
 * @author		Nikita Kuznetsov (ET-NiK)
 * @copyright	Copyright (c) 2013, Nikita Kuznetsov (http://hldm.org)
 * @license		http://gameap.ru/license.html
 * @link		http://gameap.ru
 * @filesource
*/
class Log_view extends CI_Controller {

	public function __construct()
    {
        parent::__construct();
		
		$this->load->database();
        $this->load->model('users');
        $this->lang->load('log_view');
        $this->load->library('pagination');
        
        if ($this->users->check_user()) {
			
			$this->load->helper('date');
			
			//Base Template
			$this->tpl_data['title'] = lang('log_view_title_index');
			$this->tpl_data['heading'] = lang('log_view_heading_index');
			$this->tpl_data['content'] = '';
			$this->tpl_data['menu'] = $this->parser->parse('menu.html', $this->tpl_data, TRUE);
			$this->tpl_data['profile'] = $this->parser->parse('profile.html', $this->users->tpl_userdata(), TRUE);
			
        }else{
            redirect('auth');
        }
    }
    
    // -----------------------------------------------------------------
    
    // Отображение информационного сообщения
	function _show_message($message = FALSE, $link = FALSE, $link_text = FALSE)
	{
		
		if (!$message) {
			$message = lang('error');
		}
		
		if (!$link) {
			$link = site_url('admin');
		}
		
		if (!$link_text) {
			$link_text = lang('back');
		}
	
		$local_tpl_data['message'] = $message;
		$local_tpl_data['link'] = $link;
		$local_tpl_data['back_link_txt'] = $link_text;
		$this->tpl_data['content'] = $this->parser->parse('info.html', $local_tpl_data, TRUE);
		$this->parser->parse('main.html', $this->tpl_data);
	}
	
	// -----------------------------------------------------------------
	
	/**
	 * Переопределение метода для page
	*/
	public function _remap($method, $params = array())
	{
		if ($method == 'page' or $method == 'index') {
			return call_user_func_array(array($this, 'index'), $params);
		}
		
		if (method_exists($this, $method)) {
			return call_user_func_array(array($this, $method), $params);
		}
		
		show_404();
	}
	
	// -----------------------------------------------------------------
	
	/**
	 * Выводит список логов
	*/
	function _list_log($limit = 100, $offset = FALSE)
	{
		$panel_logs = $this->panel_log->get_log(FALSE, $limit, $offset); // Логи сервера в админпанели
		
		if (is_array($panel_logs)) {
			$i = 0;
			foreach($panel_logs as $log) {
				$logs[$i]['log_id'] = 			$log['id'];
				$logs[$i]['log_date'] = 		unix_to_human($log['date'], TRUE, 'eu');
				$logs[$i]['log_server_id'] = 	$log['server_id'];
				$logs[$i]['log_user_name'] = 	$log['user_name'];
				$logs[$i]['log_command'] = 		$log['command'];
				$logs[$i]['log_type'] = 			$log['type'];
				
				$i ++;
			}
			return $logs;
		}
		
		return array();
	}
	
	// -----------------------------------------------------------------
	
	/**
	 * Отображение списка логов
	*/
	function index($offset = 0) 
	{
		/* Доступ только для админа */
		if (FALSE == $this->users->auth_data['is_admin']) {
			show_404();
		}
		
		/* Постраничная навигация */
		$config['base_url'] = site_url() . 'log_view/page';
		$config['total_rows'] = $this->panel_log->get_count_all_log();
		$config['per_page'] = 100;
		$config['full_tag_open'] = '<p align="center">';
		$config['full_tag_close'] = '</p>';
		
		$this->pagination->initialize($config); 
		$local_tpl_data['pagination'] = $this->pagination->create_links();
		
		/* Список логов */
		$local_tpl_data['log_list'] = $this->_list_log(100, $offset);

		$this->tpl_data['content'] .= $this->parser->parse('log_list.html', $local_tpl_data, TRUE);
		
		$this->parser->parse('main.html', $this->tpl_data);
	}
    
    // -----------------------------------------------------------------
    
    
    function view($id = FALSE)
    {
		$this->load->model('servers');
		
		/* Проверочки */
		if(!$id) {
			$this->_show_message(lang('log_view_id_param_not_specified'));
			return FALSE;
		}
				
		// Получаем содержимое лога
		$where = array('id' => $id);
		$log_list = $this->panel_log->get_log($where, 1);

		/* Если пользователь не админ, то делаем проверочки на права доступа к логам */
		if (!$this->users->auth_data['is_admin']) {
			
			/* Проверки для простых пользователей, которым
			 * есть доступ только для логов, относящийс к серверам
			*/
			
			/* Лог должен быть найден и относиться к серверу */
			if(!$log_list OR !$log_list[0]['server_id']) {
				$this->_show_message(lang('log_view_record_not_found'));
				return FALSE;
			}

			/* Если сервер, указанный в логе не найден */
			//~ if(!$this->servers->server_data) {
				//~ $this->_show_message(lang('log_view_record_not_found'));
				//~ return FALSE;
			//~ }
			
			/* Получаем права на сервер */
			$this->users->get_server_privileges($log_list[0]['server_id']);
			
			/* У пользователя на этот сервер должны быть права */
			if(!$this->users->servers_privileges['VIEW']) {
				$this->_show_message(lang('log_view_record_not_found'));
				return FALSE;
			}
			
			/* Если пользователю нельзя видеть rcon пароль */
			if(!$this->users->servers_privileges['CHANGE_RCON'] && strtolower($log_list[0]['command']) == 'rcon_password') {
				$this->_show_message(lang('log_view_log_access_denied'), '/admin/server_control/main/' . $log_list[0]['server_id']);
				return FALSE;
			}
		}
		
		/* Все проверки пройдены! */
		
		/* Получаем данные сервера, если лог относится к серверу */
		if ($log_list[0]['server_id']) { $this->servers->get_server_data($log_list[0]['server_id']);}
		
		$local_tpl_data['log_id'] = $log_list[0]['id'];
		$local_tpl_data['log_date'] = unix_to_human($log_list[0]['date'], TRUE, 'eu');
		$local_tpl_data['log_type'] = $log_list[0]['type'];
		$local_tpl_data['log_command'] = $log_list[0]['command'];
		$local_tpl_data['log_user'] = $log_list[0]['user_name'];
		$local_tpl_data['log_user_name'] = $log_list[0]['user_name'];
		$local_tpl_data['server_id'] = $log_list[0]['server_id'];
		if ($this->servers->server_data) { $local_tpl_data['server_name'] = $this->servers->server_data['name'];}
		$local_tpl_data['log_msg'] = $log_list[0]['msg'];
		$local_tpl_data['log_data'] = $log_list[0]['log_data'];
		
		$this->tpl_data['content'] .= $this->parser->parse('log_info.html', $local_tpl_data, TRUE);
		
		$this->parser->parse('main.html', $this->tpl_data);
	}
	
}

/* End of file log_view.php */
/* Location: ./application/controllers/log_view.php */
